;============================================
; Context Help Demo
;
; RECIPE FOR TASTY CONTEXT HELP TIP WINDOWS:
; (Now with delicious translucent shadows!)
;
; 1. Create a list of control-to-text mapping structures ($contextHelp here),
;    each with a control ID and a corresponding string for the help window text.
;
; 2. Create the dialog with the WS_EX_CONTEXTHELP "extended" style, which puts
;    the "?" button in the upper-right corner of the dialog frame.
;
; 3. In the dialog proc, create a handler for the WM_HELP message. On receipt
;    of this message, create the help tip window. (I put this code in a
;    subroutine, DisplayCtrlHelp(), to make things a little cleaner, but the
;    code could go into the dialog proc itself.)
;
; 4. To create the help tip window, first match the ID of the control generating
;    the WM_HELP message to a control ID in the table you created in step 1
;    above. The control ID is in the iCntrlId member of the HELPINFO structure
;    which is passed to the window proc in the lParam parameter.
;    You can also use the iContextType member to determine whether the user
;    clicked on a control window (HELPINFO_WINDOW) or a menu item
;    (HELPINFO_MENU).
;
; 5. We need to get the position (on the screen) of the control window. This is
;    a little tricky. Here's how I did it:
;
; A. Use GetWindow() to get a handle to the dialog that contains the control.
;    This is done by using the GW_ENABLEDPOPUP value to find the currently
;    enabled popup of the main (parent) window. This seems to work reliably.
;
; B. Using this handle, use GetDlgItem() to get a handle to the control,
;    using the control ID.
;
;    Here's the code:
;
;	INVOKE	GetWindow, MainWinHandle, GW_ENABLEDPOPUP
;	MOV	dialogHandle, EAX
;	INVOKE	GetDlgItem, dialogHandle, [EBX + HELPINFO.iCtrlId]
;	MOV	controlHandle, EAX
;
; 6. Create the help window: make it a popup window (WS_POPUP), *not* a
;    child window (you can't create a child of the desktop). I gave mine the
;    WS_BORDER style. Do NOT use the WS_VISIBLE style. Use initial values
;    for size and position; I used 0, 0 for the position and fixed values
;    for the width and height. It doesn't really matter, since the window
;    isn't yet visible, and it will be repositioned and sized shortly.
;
; 7. OK, so now we need to size and position the window. I wanted each little
;    help-tip window to be automagically sized to the text block it contains,
;    so I first get the size of this block, in the subroutine MeasureText().
;    This routine uses GetTextExtentPoint32() to measure each line of text.
;    By "line" I mean a single line, not a text block containing carriage
;    returns and line feeds, so you need to scan each block of text, break it
;    into lines, measure each line, and determine the overall width (maximum
;    line length) and height (sum of line heights).
;    Before you call GetTextExtentPoint32(), you need to set up the proper
;    "environment" in the window so you get the correct measurements. This
;    means:
;
;	o Getting a device context for the window (GetDC() )
;	o Selecting the font you'll use for the text into the DC
;
;    You also need to get the location on the screen of the control window,
;    using GetWindowRect().
;
; 7. Now we know how big the text block is, so we can resize and re-
;    position the window. Here's how:
;
;	o x-position = x0 + wc/2 - wt/2 - wp/2
;	  where		x0 = x-position of control
;			wc = width of control
;			wt = width of text
;			wp = width of text padding
;
;	o y-position = y0 + hc/2
;	  where		y0 = y-position of control
;			hc = height of control
;
;    NOTE: x0 & y0 come from the window rect from GetWindowRect().
;
;	o width = wt + wp
;	o height = ht + hp
;
;	(you can play around with the padding values to make nice-looking
;	windows)
;
; 8. Almost there. To display the text, we need 3 things:
;
;	o a pointer to the text
;	o the length of the text in bytes
;	o the rectangle in which to draw the text
;
;	(The last item is set to half the padding values for the "left"
;	and "top" members, and these values plus the width/height of
;	the text for the "right" and "bottom" members.) This centers the
;	text nicely within the window.
;
;    (I use DrawText() to display the text)
;
;    Set all of these items up as global variables. Set them in the
;    help function (DisplayCtrlHelp() ) for later use by the painting
;    code in the help-window proc.
;
;    In the help-window proc, handle the WM_PAINT message. Using these
;    global variables, draw the text in the window. Before drawing, you
;    must:
;
;	o Select the font (the same one you use to measure the text with)
;	o Select the background brush
;	o Select the pen, if needed
;	o Set the background mode for drawing
;
;    After painting, restore the DC to its previous values for each of
;    these selections.
;
; 9. Last thing: closing the window.
;    I found that playing the game "Capture The Mouse" was the best way
;    to handle this. After displaying the window (with ShowWindow() ), I
;    called SetCapture() to capture the mouse by the help window. I then
;    look for WM_LBUTTONDOWN messages in the help-window proc. Since the
;    mouse is captured, this means that any mouse click anywhere on the
;    desktop will send a message to our window proc. Upon a receipt of
;    this message, simply destroy the window. It's that easy. This seems
;    to work identically to the context help implemented by MFC classes.
;
; Addendum: Adding Window Transparency
;
; Easy-peasy:
;
; 1. Create the window you want to be transparent with the WS_EX_LAYERED
;    "extended" style. (Cannot be a child window.)
; 2. Use SetLayeredWindowAttributes() to set the transparent color (key)
;    and the transparency level (0-255).
; 3. When you paint with the color selected above, that color will be
;    transparent to the extent you gave.
;
;============================================


	include \masm32\include\masm32rt.inc


;============================================
; Defines, macros, prototypes, etc.
;============================================

$contextListPtr		TYPEDEF PTR $contextHelp

WinMain			PROTO :DWORD
DisplayCtrlHelp		PROTO helpinfoPtr:DWORD, contextListPtr:$contextListPtr
DialogProc		PROTO hWin:DWORD, uMsg:DWORD, wParam:DWORD, lParam:DWORD
HelpWinProc		PROTO hWin:DWORD, uMsg:DWORD, wParam:DWORD, lParam:DWORD
MeasureText		PROTO hDC:HDC, textPtr:DWORD

$mainWindowWidth	EQU 700
$mainWindowHeight	EQU 300

$theMenu		EQU 700
$theMenuItem		EQU 1020
$theDialog		EQU 1400

$helpFontSize		EQU 8

$initialHelpWinWidth	EQU 100
$initialHelpWinHeight	EQU 20
$xPadding		EQU 10
$yPadding		EQU 8
$shadowOffsetX		EQU 5
$shadowOffsetY		EQU 5

$contextHelp		STRUCT
  ID			DD ?
  textPtr		DD ?
$contextHelp		ENDS

;===== Dialog control IDs: =====
$dlgStaffAbove		EQU 1401
$dlgStaffBelow		EQU 1402
$dlgStaffWidth		EQU 1403
$dlgStaffSpacing	EQU 1404
$dlgBarlineSpacing	EQU 1405
$dlgSpaceAfterClef	EQU 1406
$dlgSpaceAfterSigs	EQU 1407
$dlgSpaceBeforeTimeSig	EQU 1408
$dlgSpaceBetweenNums	EQU 1409
$dlgSpaceBetweenSyms	EQU 1410
$dlgLegerLines		EQU 1411
$dlgFermataPos		EQU 1412
$dlgFermataSpace	EQU 1413
$dlgAccSpacing		EQU 1414

;===== Window styles: =====
$mainWinAttrs		EQU WS_OVERLAPPEDWINDOW OR WS_VISIBLE
$helpWinAttrs		EQU WS_POPUP OR WS_BORDER
$shadowWinAttrs		EQU WS_POPUP

;===== Window background colors: =====
$bkRED			EQU 254
$bkGRN			EQU 243
$bkBLU			EQU 199

$medGray		EQU 127

$BkColor		EQU $bkRED OR ($bkGRN SHL 8) OR ($bkBLU SHL 16)

$medGrayColor		EQU $medGray OR ($medGray SHL 8) OR ($medGray SHL 16)

$shadowOpacity		EQU 100			;<50%

;============================================
; HERE BE DATA
;============================================
.data

InstanceHandle		HWND ?
MainWinHandle		HWND ?
DialogHandle		HWND ?
HelpWinHandle		HWND ?
ShadowWinHandle		HWND ?

HelpFontHandle		HFONT ?
BkBrush			HBRUSH ?
ShadowBrush		HBRUSH ?
BlackPen		HGDIOBJ ?

TextPtr			DD ?
TextLength		DD ?
TextRect		RECT <>

ContextHelpList		$contextHelp <$dlgStaffAbove, StaffAboveText>
			$contextHelp <$dlgStaffBelow, StaffBelowText>
			$contextHelp <$dlgStaffWidth, StaffWidthText>
			$contextHelp <$dlgStaffSpacing, StaffSpacingText>
			$contextHelp <$dlgBarlineSpacing, BarlineSpacingText>
			$contextHelp <$dlgSpaceAfterClef, SpaceAfterClefText>
			$contextHelp <$dlgSpaceAfterSigs, SpaceAfterSigsText>
			$contextHelp <$dlgSpaceBeforeTimeSig, SpaceBeforeTimeSigText>
			$contextHelp <$dlgSpaceBetweenNums, SpaceBetweenNumsText>
			$contextHelp <$dlgSpaceBetweenSyms, SpaceBetweenSymsText>
			$contextHelp <$dlgLegerLines, LegerLinesText>
			$contextHelp <$dlgFermataPos, FermataPosText>
			$contextHelp <$dlgFermataSpace, FermataSpaceText>
			$contextHelp <$dlgAccSpacing, AccSpacingText>
			$contextHelp <-1, 0>		;Marks end o'list.

CCinitStruct		INITCOMMONCONTROLSEX <SIZEOF INITCOMMONCONTROLSEX, ICC_STANDARD_CLASSES>

OK2Paint		DW FALSE

MainClassName		DB "contexthelpDemo", 0
HelpClassName		DB "contexthelp", 0

MainTitleText		DB "Context Help Demo", 0

HelpFontName		DB "MS Sans Serif", 0

StaffAboveText		DB "This sets the amount of space", 13, 10
			DB "above the top line of the staff.", 0
StaffBelowText		DB "This sets the amount of space", 13, 10
			DB "below the bottom line of the staff.", 0
StaffWidthText		DB "This sets the total", 13, 10
			DB "width of the staff.", 0
StaffSpacingText	DB "This sets the spacing", 13, 10
			DB "between staff lines.", 0
BarlineSpacingText	DB "This sets the space around barlines", 0
SpaceAfterClefText	DB "This sets the space after", 13, 10
			DB "the clef of each staff.", 0
SpaceAfterSigsText	DB "This sets the spacing after the last", 13, 10
			DB "signature (key/time) on each staff.", 0
SpaceBeforeTimeSigText	DB "This sets the space before the time signature.", 0
SpaceBetweenNumsText	DB "This sets the horizontal spacing between", 13, 10
			DB "numerals in the time signature.", 0
SpaceBetweenSymsText	DB "This sets the spacing between symbols", 13, 10
			DB "(notes/rests, etc.) on the staff.", 0
LegerLinesText		DB "This sets the length of leger lines.", 0
FermataPosText		DB "This sets the default vertical position", 13, 10
			DB "of fermatas above the staff.", 0
FermataSpaceText	DB "This sets the minimum vertical spacing between", 13, 10
			DB "a symbol (note or rest) and a fermata.", 0
AccSpacingText		DB "This sets the spacing between accidentals", 13, 10
			DB "(sharps, flats and naturals) and notes.", 0


;============================================
; CODE LIVES HERE
;============================================
.code


start:
	INVOKE	GetModuleHandle, NULL
	MOV	InstanceHandle, EAX

	INVOKE	WinMain, EAX
	INVOKE	ExitProcess, EAX

	INVOKE	InitCommonControlsEx, OFFSET CCinitStruct

;====================================================================
; Mainline proc
;====================================================================

WinMain	PROC hInst:DWORD
	LOCAL	wc:WNDCLASSEX, msg:MSG, brush:HBRUSH, wX:DWORD, wY:DWORD, lf:LOGFONT

; Set up help window font:
	PUSH	EDI
	LEA	EDI, lf
	MOV	ECX, SIZEOF LOGFONT
	XOR	AL, AL
	REP	STOSB			;Clear structure
; Set fields in LOGFONT struct:
	MOV	lf.lfHeight, $helpFontSize
	MOV	lf.lfWeight, FW_NORMAL
	MOV	lf.lfCharSet, ANSI_CHARSET
	MOV	lf.lfQuality,  ANTIALIASED_QUALITY
	LEA	ESI, HelpFontName
	LEA	EDI, lf.lfFaceName
	MOV	ECX, SIZEOF HelpFontName - 1
	REP	MOVSB
	POP	EDI

	INVOKE	CreateFontIndirect, ADDR lf
	MOV	HelpFontHandle, EAX

	INVOKE	GetStockObject, BLACK_PEN
	MOV	BlackPen, EAX

; Create  brush to set background color:
	INVOKE	CreateSolidBrush, $BkColor
	MOV	BkBrush, EAX
	INVOKE	CreateSolidBrush, $medGrayColor
	MOV	ShadowBrush, EAX

; Register class for parent window:
	MOV	wc.cbSize, SIZEOF WNDCLASSEX
	MOV	wc.style,  CS_HREDRAW or CS_VREDRAW or CS_BYTEALIGNWINDOW
	MOV	wc.lpfnWndProc, OFFSET MainWindowProc
	MOV	wc.cbClsExtra, NULL
	MOV	wc.cbWndExtra, NULL
	MOV	EAX, hInst
	MOV	wc.hInstance, EAX
	MOV	EAX, BkBrush
	MOV	wc.hbrBackground, EAX
	MOV	wc.lpszMenuName,  NULL
	MOV	wc.lpszClassName, OFFSET MainClassName
	MOV	wc.hIcon, NULL
	INVOKE	LoadCursor, NULL, IDC_ARROW
	MOV	wc.hCursor, EAX
	MOV	wc.hIconSm, 0
	INVOKE	RegisterClassEx, ADDR wc

; Register class for help windows:
	MOV	wc.lpfnWndProc, OFFSET HelpWinProc
	MOV	wc.lpszClassName, OFFSET HelpClassName
	INVOKE	RegisterClassEx, ADDR wc

	INVOKE	GetSystemMetrics, SM_CXSCREEN
	MOV	EDX, $mainWindowWidth
	CALL	CenterDim
	MOV	wX, EAX
	INVOKE	GetSystemMetrics, SM_CYSCREEN
	MOV	EDX, $mainWindowHeight
	CALL	CenterDim
	MOV	wY, EAX

	INVOKE	LoadMenu, hInst, $theMenu

; Create our main window:
	INVOKE	CreateWindowEx, WS_EX_OVERLAPPEDWINDOW, ADDR MainClassName, ADDR MainTitleText,
		$mainWinAttrs, wX, wY, $mainWindowWidth, $mainWindowHeight,
		NULL, EAX, hInst, NULL
	MOV	MainWinHandle, EAX

;============= Message loop ===================
msgloop:
	INVOKE	GetMessage, ADDR msg, NULL, 0, 0
	OR	EAX, EAX				;EAX = 0 = exit
	JZ	exit99
	INVOKE	TranslateMessage, ADDR msg
	INVOKE	DispatchMessage, ADDR msg
	JMP	msgloop

exit99:	MOV	EAX, msg.wParam
	RET

WinMain		ENDP


;====================================================================
; Main Window Proc
;====================================================================

MainWindowProc	PROC hWin:DWORD, uMsg:DWORD, wParam:DWORD, lParam:DWORD

	MOV	EAX, uMsg
	CMP	EAX, WM_COMMAND
	JE	dowincmd
	CMP	EAX, WM_CLOSE
	JE	doclose

dodefault:
; use DefWindowProc for all other messages:
	INVOKE	DefWindowProc, hWin, uMsg, wParam, lParam
	RET

dowincmd:
	CMP	WORD PTR wParam, $theMenuItem
	JNE	dodefault

do_dialog:
	INVOKE	DialogBoxParam, InstanceHandle, $theDialog, hWin,
		OFFSET DialogProc, 0
	XOR	EAX, EAX
	RET

doclose:
	INVOKE	PostQuitMessage, NULL
	MOV	EAX, TRUE
	RET


MainWindowProc	ENDP


;====================================================================
; Dialog Window Proc
;====================================================================

DialogProc	PROC hWin:DWORD, uMsg:DWORD, wParam:DWORD, lParam:DWORD
		local buffer[100]:byte

	MOV	EAX, uMsg
	CMP	EAX, WM_HELP
	JE	do_help
	CMP	EAX, WM_CLOSE
	JE	do_close
	CMP	EAX, WM_INITDIALOG
	JE	do_init
	CMP	EAX, WM_COMMAND
	JE	do_command
dodefault:
	XOR	EAX, EAX
	RET

do_help:
	INVOKE	DisplayCtrlHelp, lParam, OFFSET ContextHelpList
do_init:
	MOV	EAX, TRUE
	RET

do_command:
	CMP	WORD PTR wParam, IDOK
	JNE	dodefault
do_close:
	INVOKE	EndDialog, hWin, 0
	MOV	EAX, TRUE
	RET

DialogProc	ENDP


;====================================================================
; Help Window Proc
;====================================================================

HelpWinProc	PROC hWin:DWORD, uMsg:DWORD, wParam:DWORD, lParam:DWORD
		LOCAL hDC:HDC, ps:PAINTSTRUCT, oldFont:HFONT, oldBrush:HBRUSH, oldPen:HPEN
		LOCAL oldBkMode:DWORD, gpRect:RECT
		local buffer[100]:byte

	MOV	EAX, uMsg
	CMP	EAX, WM_PAINT
	JE	do_paint
	CMP	EAX, WM_LBUTTONDOWN
	JE	do_close
	CMP	EAX, WM_CLOSE
	JE	do_close

dodefault:
	INVOKE	DefWindowProc, hWin, uMsg, wParam, lParam
	RET

do_paint:
	INVOKE	BeginPaint, hWin, ADDR ps
	MOV	hDC, EAX
	MOV	EAX, hWin
	CMP	EAX, ShadowWinHandle
	JE	do_shadow
	INVOKE	SelectObject, hDC, HelpFontHandle
	MOV	oldFont, EAX
	INVOKE	SelectObject, hDC, BkBrush
	MOV	oldBrush, EAX
	INVOKE	SelectObject, hDC, BlackPen
	MOV	oldPen, EAX
	INVOKE	SetBkMode, hDC, TRANSPARENT
	MOV	oldBkMode, EAX
	INVOKE	DrawText, hDC, TextPtr, TextLength, ADDR TextRect, DT_LEFT OR DT_WORDBREAK
	INVOKE	SetBkMode, hDC, oldBkMode
	INVOKE	SelectObject, hDC, oldPen
	INVOKE	SelectObject, hDC, oldBrush
	INVOKE	SelectObject, hDC, oldFont
end_paint:
	INVOKE	EndPaint, hWin, ADDR ps
	XOR	EAX, EAX
	RET

do_shadow:
	INVOKE	GetClientRect, hWin, ADDR gpRect
	INVOKE	FillRect, hDC, ADDR gpRect, ShadowBrush
	JMP	end_paint			;Recycle some code.

do_close:
	CALL	ReleaseCapture
; Explicitly remove both windows:
	INVOKE	DestroyWindow, HelpWinHandle
	INVOKE	DestroyWindow, ShadowWinHandle
	XOR	EAX, EAX
	RET


HelpWinProc	ENDP


;====================================================================
; DisplayCtrlHelp (helpinfoPtr, contextListPtr)
;
; Creates the help-tip window and fills it with text.
; Uses the control ID in the HELPINFO structure passed in to find
; the appropriate help text. Sizes the window to fit the block of
; text and positions the window just under the control the user
; clicked on.
;
; Captures the mouse, so someone needs to call ReleaseCapture() to
; release the mouse.
;
; Returns nothing.
;====================================================================

DisplayCtrlHelp	PROC helpinfoPtr:DWORD, contextListPtr:$contextListPtr
		LOCAL hDC:HDC, sstr:_SIZE, rect1:RECT, rect2:RECT, oldFont:HFONT
		LOCAL xPos:DWORD, yPos:DWORD, dialogHandle:HWND, controlHandle:HWND
		LOCAL textWidth:DWORD, textHeight:DWORD, winWidth:DWORD, winHeight:DWORD
		local buffer[100]:byte

	PUSH	EBX
	MOV	EBX, helpinfoPtr

; Make sure help is for a window, not a menu item:
	CMP	[EBX + HELPINFO.iContextType], HELPINFO_WINDOW
	JNE	dch99

; Match ID of control to HELPINFO ID:
	MOV	EAX, [EBX + HELPINFO.iCtrlId]
	MOV	EDX, contextListPtr
dch10:	CMP	[EDX + $contextHelp.ID], -1		;Look for end o'list.
	JE	dch99					;Hit it, bye-bye.
	CMP	EAX, [EDX + $contextHelp.ID]		;Match on ID?
	JE	dch40					;  Yep, show it.
	ADD	EDX, SIZEOF $contextHelp		;Next item in list.
	JMP	dch10

dch40:	MOV	EAX, [EDX + $contextHelp.textPtr]
	MOV	TextPtr, EAX				;Store pointer to text
	CALL	strlen
	MOV	TextLength, EAX				;  and length globally.

; Create the shadow window first as a popup on the desktop:
	INVOKE	CreateWindowEx, WS_EX_LEFT OR WS_EX_LAYERED, ADDR HelpClassName, NULL,
		$shadowWinAttrs, 0, 0, $initialHelpWinWidth, $initialHelpWinHeight,
		0, NULL, InstanceHandle, NULL
	MOV	ShadowWinHandle, EAX
	INVOKE	SetLayeredWindowAttributes, EAX, $medGrayColor, $shadowOpacity, LWA_ALPHA

; Create the help window:
	INVOKE	CreateWindowEx, WS_EX_LEFT, ADDR HelpClassName, NULL,
		$helpWinAttrs, 0, 0, $initialHelpWinWidth, $initialHelpWinHeight,
		0, NULL, InstanceHandle, NULL
	MOV	HelpWinHandle, EAX

	INVOKE	GetDC, HelpWinHandle
	MOV	hDC, EAX
	INVOKE	SelectObject, hDC, HelpFontHandle
	MOV	oldFont, EAX
	INVOKE	MeasureText, hDC, TextPtr
	MOV	textWidth, EAX
	MOV	textHeight, EDX

; Set up text rectangle:
	MOV	TextRect.left, $xPadding/2
	ADD	EAX, $xPadding
	MOV	TextRect.right, EAX
	MOV	TextRect.top, $yPadding/2
	ADD	EDX, $yPadding
	MOV	TextRect.bottom, EDX

	INVOKE	SelectObject, hDC, oldFont
	INVOKE	ReleaseDC, HelpWinHandle, hDC

; Get position of help window based on control position:
	INVOKE	GetWindow, MainWinHandle, GW_ENABLEDPOPUP
	MOV	dialogHandle, EAX
	INVOKE	GetDlgItem, dialogHandle, [EBX + HELPINFO.iCtrlId]
	MOV	controlHandle, EAX
	INVOKE	GetWindowRect, controlHandle, ADDR rect2

; Set xPos = control (x0) + (control width / 2) - (text width / 2) - (padding / 2):
	MOV	EAX, rect2.left
	MOV	EDX, rect2.right
	SUB	EDX, EAX			;EDX = control width.
	SHR	EDX, 1				;1/2 width.
	ADD	EAX, EDX
	MOV	EDX, textWidth
	SHR	EDX, 1				;1/2 text width.
	SUB	EAX, EDX
	SUB	EAX, $xPadding/2
	MOV	xPos, EAX

; Set yPos = control (y0) + (text height / 2):
	MOV	EAX, rect2.top
	MOV	EDX, textHeight
	SHR	EDX, 1
	SUB	EAX, EDX
	MOV	yPos, EAX

; Set window width = text width + x-padding, height = text height + y-padding:
	MOV	EAX, textWidth
	ADD	EAX, $xPadding
	MOV	winWidth, EAX
	MOV	EAX, textHeight
	ADD	EAX, $yPadding
	MOV	winHeight, EAX

; Move windows to new size & position and show them (force painting):
	MOV	EAX, xPos
	ADD	EAX, $shadowOffsetX
	MOV	EDX, yPos
	ADD	EDX, $shadowOffsetY
	INVOKE	MoveWindow, ShadowWinHandle, EAX, EDX, winWidth, winHeight, TRUE
	INVOKE	ShowWindow, ShadowWinHandle, SW_NORMAL
	INVOKE	MoveWindow, HelpWinHandle, xPos, yPos, winWidth, winHeight, TRUE
	INVOKE	ShowWindow, HelpWinHandle, SW_NORMAL

; Capture the mouse so we pick up clicks anywhere on the screen:
	INVOKE	SetCapture, HelpWinHandle

dch99:	POP	EBX
	RET

DisplayCtrlHelp	ENDP


;====================================================================
; MeasureText (hDC, textPtr)
;
; Measures width & height of text block, using passed-in HDC.
;
; Returns:
;	EAX: text width
;	EDX: text height
;====================================================================

MeasureText	PROC hDC:HDC, textPtr:DWORD
		LOCAL maxX:DWORD, height:DWORD, buffer[200]:BYTE, szst:_SIZE
		LOCAL done:DWORD

	PUSH	ESI
	PUSH	EDI

	MOV	maxX, 0
	MOV	height, 0
	MOV	done, FALSE

	MOV	ESI, textPtr

mt3:	LEA	EDI, buffer
mt5:	LODSB
	OR	AL, AL				;End o'text?
	JZ	mt8
	CMP	AL, 10				;Skip linefeeds.
	JE	mt5
	CMP	AL, 13				;Break on carriage returns.
	JE	mt10
	STOSB					;Store character.
	JMP	mt5

mt8:	MOV	done, TRUE

; <CR> hit--process line of text:
mt10:	MOV	BYTE PTR [EDI], 0		;Make ASCIIZ.
	LEA	EAX, buffer
	CALL	strlen
	MOV	EDX, EAX
	INVOKE	GetTextExtentPoint32, hDC, ADDR buffer, EDX, ADDR szst

	MOV	EAX, szst.y
	ADD	height, EAX
; Compare string length to greatest so far:
	MOV	EAX, szst.x
	CMP	EAX, maxX
	JBE	@F
	MOV	maxX, EAX
@@:	CMP	done, TRUE
	JE	@F
	JMP	mt3				;Go back for more.

@@:	MOV	EAX, maxX
	MOV	EDX, height
	POP	EDI
	POP	ESI
	RET

MeasureText	ENDP



;====================================================================
; strlen()
;
; On entry,
;   EAX--> string to be measured
;====================================================================

strlen		PROC

	XOR	EDX, EDX			;Len. counter
sl10:	CMP	BYTE PTR [EAX], NULL
	JE	sl88
	INC	EDX
	INC	EAX
	JMP	sl10	
sl88:	MOV	EAX, EDX
	RET

strlen		ENDP


;====================================================================
; Displays last Windows system error (assuming there was one), using
; GetLastError(). Formats message and displays it using MessageBox().
;
; Returns last error code.
;====================================================================

ShowLastError	PROC
		LOCAL buffer[200]:BYTE

	CALL	GetLastError
	MOV	EDX, EAX
	PUSH	EAX
	INVOKE	FormatMessage, FORMAT_MESSAGE_FROM_SYSTEM OR FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL, EDX, 0, ADDR buffer, SIZEOF buffer, NULL
	INVOKE	MessageBox, 0, ADDR buffer, NULL, MB_OK
	POP	EAX
	RET

ShowLastError	ENDP


;====================================================================
; CenterDim
;
; Returns half screen dimension (X or Y) minus half window dimension
;
; On entry,
;	EAX = screen dimension
;	EDX = window dimension
;
; Returns:
;	sdim/2 - wdim/2
;====================================================================

CenterDim	PROC

	SHR	EAX, 1				;divide screen dimension by 2
	SHR	EDX, 1				;divide window dimension by 2
	SUB	EAX, EDX
	RET					;Return w/# in EAX

CenterDim	ENDP


	END	start

